@@ -10,8 +10,7 @@ require 'commonmarker/version'
 
 begin
   require 'awesome_print'
-rescue LoadError; end
-
+rescue LoadError; end # rubocop:disable Lint/SuppressedException
 module CommonMarker
   # Public:  Parses a Markdown string into an HTML string.
   #
@@ -21,7 +20,8 @@ module CommonMarker
   #
   # Returns a {String} of converted HTML.
   def self.render_html(text, options = :DEFAULT, extensions = [])
-    fail TypeError, "text must be a String; got a #{text.class}!" unless text.is_a?(String)
+    raise TypeError, "text must be a String; got a #{text.class}!" unless text.is_a?(String)
+
     opts = Config.process_options(options, :render)
     text = text.encode('UTF-8')
     html = Node.markdown_to_html(text, opts, extensions)
@@ -36,7 +36,8 @@ module CommonMarker
   #
   # Returns the `document` node.
   def self.render_doc(text, options = :DEFAULT, extensions = [])
-    fail TypeError, "text must be a String; got a #{text.class}!" unless text.is_a?(String)
+    raise TypeError, "text must be a String; got a #{text.class}!" unless text.is_a?(String)
+
     opts = Config.process_options(options, :parse)
     text = text.encode('UTF-8')
     Node.parse_document(text, text.bytesize, opts, extensions)
