@@ -11,7 +11,7 @@ module CommonMarker
         PP.pp(self, +'', Float::INFINITY)
       end
 
-      # @param [PrettyPrint] pp
+      # @param printer [PrettyPrint] pp
       def pretty_print(printer)
         printer.group(PP_INDENT_SIZE, "#<#{self.class}(#{type}):", '>') do
           printer.breakable
@@ -27,11 +27,9 @@ module CommonMarker
             list_tight
             fence_info
           ].map do |name|
-            begin
-              [name, __send__(name)]
-            rescue NodeError
-              nil
-            end
+            [name, __send__(name)]
+          rescue NodeError
+            nil
           end.compact
 
           printer.seplist(attrs) do |name, value|
